@@ -235,14 +235,16 @@ wb_to_df <- function(
   # If no dims are requested via named_region, simply construct them from min
   # and max columns and row found on worksheet
   # TODO it would be useful to have both named_region and dims?
+  has_dims <- TRUE
   if (missing(named_region) && missing(dims)) {
+    has_dims <- FALSE
 
     sd <- wb$worksheets[[sheet]]$sheet_data$cc[c("row_r", "c_r")]
-    sd$row <- as.integer(sd$row_r)
-    sd$col <- col2int(sd$c_r)
+    row <- range(as.integer(unique(sd$row_r)))
+    col <- range(col2int(unique(sd$c_r)))
 
-    dims <- paste0(int2col(min(sd$col)), min(sd$row), ":",
-                   int2col(max(sd$col)), max(sd$row))
+    dims <- paste0(int2col(col[1]), row[1], ":",
+                   int2col(col[2]), row[2])
 
   }
 
@@ -336,7 +338,7 @@ wb_to_df <- function(
   keep_rows <- keep_rows[keep_rows %in% rnams]
 
   # reduce data to selected cases only
-  if (length(keep_rows) && length(keep_cols))
+  if (has_dims && length(keep_rows) && length(keep_cols))
     cc <- cc[cc$row_r %in% keep_rows & cc$c_r %in% keep_cols, ]
 
   cc$val <- NA_character_
@@ -430,19 +432,22 @@ wb_to_df <- function(
   }
 
   # remaining values are numeric?
-  if (any(sel <- is.na(cc$typ))) {
+  if (any(cc_tab %in% c("n", ""))) {
+    sel <- which(is.na(cc$typ))
     cc$val[sel] <- cc$v[sel]
     cc$typ[sel] <- "n"
   }
 
   if (show_formula) {
 
-    if (any(cc$f_t == "shared")) {
+    if (any(grepl("shared", cc$f_attr))) {
+
+      f_t <- rbindlist(xml_attr(paste0("<f ", cc$f_attr, "/>"), "f"))$t
       # depending on the sheet, this might require updates to many cells
       # TODO reduce this to cells, that are part of `cc`. Currently we
       # might waste time, updating cells that are not visible to the user
       cc_shared <- wb$worksheets[[sheet]]$sheet_data$cc
-      cc_shared <- cc_shared[cc_shared$f_t == "shared", ]
+      cc_shared <- cc_shared[f_t == "shared", ]
       cc <- shared_as_fml(cc, cc_shared)
     }
 
@@ -484,7 +489,7 @@ wb_to_df <- function(
   zz$cols <- match(cc$c_r, colnames(z)) - 1L
   zz$rows <- match(cc$row_r, rownames(z)) - 1L
 
-  zz <- zz[order(zz[, "cols"], zz[, "rows"]), ]
+  # zz <- zz[order(zz[, "cols"], zz[, "rows"]), ]
   if (any(zz$val == "", na.rm = TRUE)) zz <- zz[zz$val != "", ]
   long_to_wide(z, tt, zz)
 
