@@ -1,5 +1,5 @@
-#include "openxlsx2.h"
 #include <algorithm>
+#include "openxlsx2.h"
 
 // For R-devel 4.3 character length on Windows was modified. This caused an
 // error when passing large character strings to file.exist() when called in
@@ -16,12 +16,11 @@
 //' @noRd
 // [[Rcpp::export]]
 bool to_long(std::string path) {
- return path.size() > R_PATH_MAX;
+  return path.size() > R_PATH_MAX;
 }
 
 // [[Rcpp::export]]
 SEXP openxlsx2_type(SEXP x) {
-
   const SEXP names = Rf_getAttrib(x, R_NamesSymbol);
   R_xlen_t ncol = Rf_length(x);
 
@@ -29,7 +28,6 @@ SEXP openxlsx2_type(SEXP x) {
   if (!Rf_isNull(names)) type.attr("names") = names;
 
   for (R_xlen_t i = 0; i < ncol; ++i) {
-
     // check if dim != NULL
     SEXP z;
     if (Rf_isNull(names)) {
@@ -41,123 +39,137 @@ SEXP openxlsx2_type(SEXP x) {
     SEXP Rclass = Rf_getAttrib(z, R_ClassSymbol);
 
     switch (TYPEOF(z)) {
-
-    // logical
-    case LGLSXP:
-      if (Rf_isNull(Rclass)) {
-        type[i] = logical; // logical
-      } else {
-        type[i] = factor; // probably some custom class
-      };
-      break;
-
-      // character, formula, hyperlink, array_formula
-    case CPLXSXP:
-    case STRSXP:
-      if (Rf_inherits(z, "formula")) {
-        type[i] = formula;
-      } else if (Rf_inherits(z, "hyperlink")) {
-        type[i] = hyperlink;
-      } else if (Rf_inherits(z, "array_formula")) {
-        type[i] = array_formula;
-      } else if (Rf_inherits(z, "cm_formula")) {
-        type[i] = cm_formula;
-      } else {
-        type[i] = character;
-      }
-      break;
-
-      // raw, integer, numeric, Date, POSIXct, accounting,
-      //  percentage, scientific, comma
-    case RAWSXP:
-    case INTSXP:
-    case REALSXP: {
-      if (Rf_inherits(z, "Date")) {
-        type[i] = short_date;
-      } else if (Rf_inherits(z, "POSIXct")) {
-        type[i] = long_date;
-      } else if (Rf_inherits(z, "accounting")) {
-        type[i] = accounting;
-      } else if (Rf_inherits(z, "percentage")) {
-        type[i] = percentage;
-      } else if (Rf_inherits(z, "scientific")) {
-        type[i] = scientific;
-      } else if (Rf_inherits(z, "comma")) {
-        type[i] = comma;
-      } else if (Rf_inherits(z, "factor") || !Rf_isNull(Rf_getAttrib(z, Rf_install("labels")))) {
-        type[i] = factor;
-      } else if (Rf_inherits(z, "hms")) {
-        type[i] = hms_time;
-      } else if (Rf_inherits(z, "currency")) {
-        type[i] = currency;
-      } else {
+      // logical
+      case LGLSXP:
         if (Rf_isNull(Rclass)) {
-          type[i] = numeric; // numeric and integer
+          type[i] = logical;  // logical
         } else {
-          type[i] = factor; // probably some custom class
+          type[i] = factor;  // probably some custom class
+        };
+        break;
+
+        // character, formula, hyperlink, array_formula
+      case CPLXSXP:
+      case STRSXP:
+        if (Rf_inherits(z, "formula")) {
+          type[i] = formula;
+        } else if (Rf_inherits(z, "hyperlink")) {
+          type[i] = hyperlink;
+        } else if (Rf_inherits(z, "array_formula")) {
+          type[i] = array_formula;
+        } else if (Rf_inherits(z, "cm_formula")) {
+          type[i] = cm_formula;
+        } else {
+          type[i] = character;
         }
+        break;
+
+        // raw, integer, numeric, Date, POSIXct, accounting,
+        //  percentage, scientific, comma
+      case RAWSXP:
+      case INTSXP:
+      case REALSXP: {
+        if (Rf_inherits(z, "Date")) {
+          type[i] = short_date;
+        } else if (Rf_inherits(z, "POSIXct")) {
+          type[i] = long_date;
+        } else if (Rf_inherits(z, "accounting")) {
+          type[i] = accounting;
+        } else if (Rf_inherits(z, "percentage")) {
+          type[i] = percentage;
+        } else if (Rf_inherits(z, "scientific")) {
+          type[i] = scientific;
+        } else if (Rf_inherits(z, "comma")) {
+          type[i] = comma;
+        } else if (Rf_inherits(z, "factor") || !Rf_isNull(Rf_getAttrib(z, Rf_install("labels")))) {
+          type[i] = factor;
+        } else if (Rf_inherits(z, "hms")) {
+          type[i] = hms_time;
+        } else if (Rf_inherits(z, "currency")) {
+          type[i] = currency;
+        } else {
+          if (Rf_isNull(Rclass)) {
+            type[i] = numeric;  // numeric and integer
+          } else {
+            type[i] = factor;  // probably some custom class
+          }
+        }
+        break;
+      }
+
+      case VECSXP:
+        type[i] = list;
+        break;
+
+      // whatever is not covered from above
+      default: {
+        type[i] = character;
+        break;
       }
-      break;
     }
-
-    case VECSXP:
-      type[i] = list;
-      break;
-
-
-    // whatever is not covered from above
-    default: {
-      type[i] = character;
-      break;
-    }
-
-    }
-
   }
 
   return type;
 }
 
-
 // [[Rcpp::export]]
 Rcpp::IntegerVector col_to_int(Rcpp::CharacterVector x) {
-
   // This function converts the Excel column letter to an integer
   R_xlen_t n = static_cast<R_xlen_t>(x.size());
-
-  std::string a;
+  std::unordered_map<std::string, int> col_map;
   Rcpp::IntegerVector colNums(n);
 
-  for (R_xlen_t i = 0; i < n; i++) {
-    a = x[i];
+  for (R_xlen_t i = 0; i < n; ++i) {
+    std::string a = Rcpp::as<std::string>(x[i]);
 
     // check if the value is digit only, if yes, add it and continue the loop
     // at the top. This avoids slow:
     // suppressWarnings(isTRUE(as.character(as.numeric(x)) == x))
-    if (std::all_of(a.begin(), a.end(), ::isdigit))
-    {
+    if (std::all_of(a.begin(), a.end(), ::isdigit)) {
       colNums[i] = std::stoi(a);
       continue;
     }
 
-    // return index from column name
-    colNums[i] = cell_to_colint(a);
+    // Check if the column name is already in the map
+    if (col_map.find(a) != col_map.end()) {
+      colNums[i] = col_map[a];
+    } else {
+      // Compute the integer value and store it in the map
+      int col_int = cell_to_colint(a);
+      col_map[a] = col_int;
+      colNums[i] = col_int;
+    }
   }
 
   return colNums;
-
 }
 
 // [[Rcpp::export]]
-std::string ox_int_to_col(int32_t cell) {
-  uint32_t cell_u32 = static_cast<uint32_t>(cell);
-  return int_to_col(cell_u32);
+Rcpp::CharacterVector ox_int_to_col(Rcpp::NumericVector x) {
+  R_xlen_t n = static_cast<R_xlen_t>(x.size());
+  Rcpp::CharacterVector colNames(n);
+  std::unordered_map<uint32_t, std::string> cache;  // Updated key type
+
+  for (R_xlen_t i = 0; i < n; ++i) {
+    uint32_t num = static_cast<uint32_t>(x[i]);
+
+    // Check if the column name is already in the cache
+    if (cache.find(num) != cache.end()) {
+      colNames[i] = cache[num];
+    } else {
+      // Compute the column name and store it in the cache
+      std::string col_name = int_to_col(num);
+      cache[num] = col_name;
+      colNames[i] = col_name;
+    }
+  }
+
+  return colNames;
 }
 
 // provide a basic rbindlist for lists of named characters
 // [[Rcpp::export]]
 SEXP rbindlist(Rcpp::List x) {
-
   R_xlen_t nn = static_cast<R_xlen_t>(x.size());
   std::vector<std::string> all_names;
 
@@ -176,8 +188,7 @@ SEXP rbindlist(Rcpp::List x) {
 
   // 1. create the list
   Rcpp::List df(kk);
-  for (R_xlen_t i = 0; i < kk; ++i)
-  {
+  for (R_xlen_t i = 0; i < kk; ++i) {
     SET_VECTOR_ELT(df, i, Rcpp::CharacterVector(Rcpp::no_init(nn)));
   }
 
@@ -193,7 +204,6 @@ SEXP rbindlist(Rcpp::List x) {
 
       Rcpp::as<Rcpp::CharacterVector>(df[mtc])[i] = Rcpp::String(values[j]);
     }
-
   }
 
   // 3. Create a data.frame
@@ -254,7 +264,6 @@ Rcpp::CharacterVector needed_cells(const std::string& range) {
 // [[Rcpp::export]]
 SEXP dims_to_df(Rcpp::IntegerVector rows, Rcpp::CharacterVector cols, Rcpp::Nullable<Rcpp::CharacterVector> filled, bool fill,
                 Rcpp::Nullable<Rcpp::IntegerVector> fcols) {
-
   R_xlen_t kk = static_cast<R_xlen_t>(cols.size());
   R_xlen_t nn = static_cast<R_xlen_t>(rows.size());
 
@@ -263,8 +272,7 @@ SEXP dims_to_df(Rcpp::IntegerVector rows, Rcpp::CharacterVector cols, Rcpp::Null
 
   // 1. create the list
   Rcpp::List df(kk);
-  for (R_xlen_t i = 0; i < kk; ++i)
-  {
+  for (R_xlen_t i = 0; i < kk; ++i) {
     if (fill)
       SET_VECTOR_ELT(df, i, Rcpp::CharacterVector(Rcpp::no_init(nn)));
     else
@@ -273,7 +281,6 @@ SEXP dims_to_df(Rcpp::IntegerVector rows, Rcpp::CharacterVector cols, Rcpp::Null
 
   if (fill) {
     if (has_filled) {
-
       std::vector<std::string> flld = Rcpp::as<std::vector<std::string>>(filled.get());
       std::unordered_set<std::string> flls(flld.begin(), flld.end());
 
@@ -288,7 +295,7 @@ SEXP dims_to_df(Rcpp::IntegerVector rows, Rcpp::CharacterVector cols, Rcpp::Null
         }
       }
 
-    } else { // insert cells into data frame
+    } else {  // insert cells into data frame
 
       std::vector<size_t> fcls;
       if (has_fcols) {
@@ -306,7 +313,7 @@ SEXP dims_to_df(Rcpp::IntegerVector rows, Rcpp::CharacterVector cols, Rcpp::Null
       }
     }
 
-  } // else return data frame filled with NA_character_
+  }  // else return data frame filled with NA_character_
 
   // 3. Create a data.frame
   df.attr("row.names") = rows;
@@ -319,12 +326,11 @@ SEXP dims_to_df(Rcpp::IntegerVector rows, Rcpp::CharacterVector cols, Rcpp::Null
 // similar to dcast converts cc dataframe to z dataframe
 // [[Rcpp::export]]
 void long_to_wide(Rcpp::DataFrame z, Rcpp::DataFrame tt, Rcpp::DataFrame zz) {
-
   R_xlen_t n = static_cast<R_xlen_t>(zz.nrow());
   R_xlen_t col = 0, row = 0;
 
-  Rcpp::IntegerVector rows = zz["rows"];
   Rcpp::IntegerVector cols = zz["cols"];
+  Rcpp::IntegerVector rows = zz["rows"];
   Rcpp::CharacterVector vals = zz["val"];
   Rcpp::CharacterVector typs = zz["typ"];
 
@@ -369,7 +375,6 @@ void wide_to_long(
     std::string c_cm,
     std::vector<std::string> dims
 ) {
-
   R_xlen_t n = static_cast<R_xlen_t>(z.nrow());
   R_xlen_t m = static_cast<R_xlen_t>(z.ncol());
   bool has_dims = static_cast<R_xlen_t>(dims.size()) == (n * m);
@@ -383,27 +388,33 @@ void wide_to_long(
   for (size_t i = 0; i < static_cast<size_t>(m); ++i) {
     scols[i] = int_to_col(static_cast<size_t>(start_col) + i);
   }
+  std::string f_attr;
 
-  bool has_refs  = refed.isNotNull();
+  bool has_refs = refed.isNotNull();
 
   std::vector<std::string> ref;
   if (has_refs) ref = Rcpp::as<std::vector<std::string>>(refed.get());
 
   int32_t in_string_nums = string_nums;
 
+  bool has_cm = zz.containsElementNamed("c_cm");
+  bool has_typ = zz.containsElementNamed("typ");
+
   // pointer magic. even though these are extracted, they just point to the
   // memory in the data frame
-  Rcpp::CharacterVector zz_row_r = Rcpp::as<Rcpp::CharacterVector>(zz["row_r"]);
-  Rcpp::CharacterVector zz_c_cm  = Rcpp::as<Rcpp::CharacterVector>(zz["c_cm"]);
-  Rcpp::CharacterVector zz_c_r   = Rcpp::as<Rcpp::CharacterVector>(zz["c_r"]);
-  Rcpp::CharacterVector zz_v     = Rcpp::as<Rcpp::CharacterVector>(zz["v"]);
-  Rcpp::CharacterVector zz_c_t   = Rcpp::as<Rcpp::CharacterVector>(zz["c_t"]);
-  Rcpp::CharacterVector zz_is    = Rcpp::as<Rcpp::CharacterVector>(zz["is"]);
-  Rcpp::CharacterVector zz_f     = Rcpp::as<Rcpp::CharacterVector>(zz["f"]);
-  Rcpp::CharacterVector zz_f_t   = Rcpp::as<Rcpp::CharacterVector>(zz["f_t"]);
-  Rcpp::CharacterVector zz_f_ref = Rcpp::as<Rcpp::CharacterVector>(zz["f_ref"]);
-  Rcpp::CharacterVector zz_typ   = Rcpp::as<Rcpp::CharacterVector>(zz["typ"]);
-  Rcpp::CharacterVector zz_r     = Rcpp::as<Rcpp::CharacterVector>(zz["r"]);
+  Rcpp::CharacterVector zz_c_cm;
+  Rcpp::CharacterVector zz_typ;
+
+  Rcpp::CharacterVector zz_row_r  = Rcpp::as<Rcpp::CharacterVector>(zz["row_r"]);
+  if (has_cm) zz_c_cm   = Rcpp::as<Rcpp::CharacterVector>(zz["c_cm"]);
+  Rcpp::CharacterVector zz_c_r    = Rcpp::as<Rcpp::CharacterVector>(zz["c_r"]);
+  Rcpp::CharacterVector zz_v      = Rcpp::as<Rcpp::CharacterVector>(zz["v"]);
+  Rcpp::CharacterVector zz_c_t    = Rcpp::as<Rcpp::CharacterVector>(zz["c_t"]);
+  Rcpp::CharacterVector zz_is     = Rcpp::as<Rcpp::CharacterVector>(zz["is"]);
+  Rcpp::CharacterVector zz_f      = Rcpp::as<Rcpp::CharacterVector>(zz["f"]);
+  Rcpp::CharacterVector zz_f_attr = Rcpp::as<Rcpp::CharacterVector>(zz["f_attr"]);
+  if (has_typ) zz_typ   = Rcpp::as<Rcpp::CharacterVector>(zz["typ"]);
+  Rcpp::CharacterVector zz_r      = Rcpp::as<Rcpp::CharacterVector>(zz["r"]);
 
   // Convert na_strings only once outside the loop.
   na_strings = inline_strings ? txt_to_is(na_strings, 0, 1, 1) : txt_to_si(na_strings, 0, 1, 1);
@@ -411,7 +422,6 @@ void wide_to_long(
   R_xlen_t idx = 0;
 
   SEXP blank_sexp      = Rf_mkChar("");
-  SEXP array_sexp      = Rf_mkChar("array");
   SEXP inlineStr_sexp  = Rf_mkChar("inlineStr");
   SEXP bool_sexp       = Rf_mkChar("b");
   SEXP expr_sexp       = Rf_mkChar("e");
@@ -423,15 +433,12 @@ void wide_to_long(
   SEXP value_sexp      = Rf_mkChar("#VALUE!");
   SEXP na_strings_sexp = Rf_mkChar(na_strings.c_str());
 
-
   for (R_xlen_t i = 0; i < m; ++i) {
-
     Rcpp::CharacterVector cvec = Rcpp::as<Rcpp::CharacterVector>(z[i]);
     const std::string& col = scols[static_cast<size_t>(i)];
     int8_t vtyp_i = static_cast<int8_t>(vtyps[static_cast<size_t>(i)]);
 
     for (R_xlen_t j = 0; j < n; ++j, ++idx) {
-
       checkInterrupt(idx);
 
       // if colname is provided, the first row is always a character
@@ -457,37 +464,35 @@ void wide_to_long(
       else
         string_nums = in_string_nums;
 
-      switch(vtyp)
-      {
-
-      case currency:
-      case short_date:
-      case long_date:
-      case accounting:
-      case percentage:
-      case scientific:
-      case comma:
-      case hms_time:
-      case numeric:
-        // v
-        SET_STRING_ELT(zz_v, pos, vals_sexp);
-        break;
-      case logical:
-        // v and c_t = "b"
-        SET_STRING_ELT(zz_v,   pos, vals_sexp);
-        SET_STRING_ELT(zz_c_t, pos, bool_sexp);
-        break;
-      case factor:
-      case character:
-
-        // test if string can be written as number
-        if (string_nums && is_double(vals)) {
+      switch (vtyp) {
+        case currency:
+        case short_date:
+        case long_date:
+        case accounting:
+        case percentage:
+        case scientific:
+        case comma:
+        case hms_time:
+        case numeric:
           // v
           SET_STRING_ELT(zz_v, pos, vals_sexp);
-          vtyp     = (string_nums == 1) ? string_num : numeric;
-        } else {
-          // check if we write sst or inlineStr
-          if (inline_strings) {
+          break;
+        case logical:
+          // v and c_t = "b"
+          SET_STRING_ELT(zz_v,   pos, vals_sexp);
+          SET_STRING_ELT(zz_c_t, pos, bool_sexp);
+          break;
+        case factor:
+        case character:
+
+          // test if string can be written as number
+          if (string_nums && is_double(vals)) {
+            // v
+            SET_STRING_ELT(zz_v, pos, vals_sexp);
+            vtyp     = (string_nums == 1) ? string_num : numeric;
+          } else {
+            // check if we write sst or inlineStr
+            if (inline_strings) {
               // is and c_t = "inlineStr"
               SET_STRING_ELT(zz_c_t, pos, inlineStr_sexp);
               SET_STRING_ELT(zz_is,  pos, Rf_mkChar(txt_to_is(vals, 0, 1, 1).c_str()));
@@ -495,32 +500,33 @@ void wide_to_long(
               // v and c_t = "s"
               SET_STRING_ELT(zz_c_t, pos, sharedStr_sexp);
               SET_STRING_ELT(zz_v,   pos, Rf_mkChar(txt_to_si(vals, 0, 1, 1).c_str()));
+            }
           }
-        }
-        break;
-      case hyperlink:
-      case formula:
-        // f and c_t = "str";
-        SET_STRING_ELT(zz_c_t, pos, string_sexp);
-        SET_STRING_ELT(zz_f,   pos, vals_sexp);
-        break;
-      case array_formula:
-        // f, f_t = "array", and f_ref
-        SET_STRING_ELT(zz_f,     pos, vals_sexp);
-        SET_STRING_ELT(zz_f_t,   pos, array_sexp);
-        SET_STRING_ELT(zz_f_ref, pos, Rf_mkChar(ref_str.c_str()));
-        break;
-      case cm_formula:
-        // c_cm, f, f_t = "array", and f_ref
-        SET_STRING_ELT(zz_c_cm,  pos, Rf_mkChar(c_cm.c_str()));
-        SET_STRING_ELT(zz_f,     pos, vals_sexp);
-        SET_STRING_ELT(zz_f_t,   pos, array_sexp);
-        SET_STRING_ELT(zz_f_ref, pos, Rf_mkChar(ref_str.c_str()));
-        break;
+          break;
+        case hyperlink:
+        case formula:
+          // f and c_t = "str";
+          SET_STRING_ELT(zz_c_t, pos, string_sexp);
+          SET_STRING_ELT(zz_f,   pos, vals_sexp);
+          break;
+        case array_formula:
+          // f, f_t = "array", and f_ref
+          f_attr = "t=\"array\" ref=\"" + ref_str + "\"";
+
+          SET_STRING_ELT(zz_f,      pos, vals_sexp);
+          SET_STRING_ELT(zz_f_attr, pos, Rf_mkChar(f_attr.c_str()));
+          break;
+        case cm_formula:
+          // c_cm, f, f_t = "array", and f_ref
+          f_attr = "t=\"array\" ref=\"" + ref_str + "\"";
+
+          SET_STRING_ELT(zz_c_cm,   pos, Rf_mkChar(c_cm.c_str()));
+          SET_STRING_ELT(zz_f,      pos, vals_sexp);
+          SET_STRING_ELT(zz_f_attr, pos, Rf_mkChar(f_attr.c_str()));
+          break;
       }
 
       if (vals_sexp == NA_STRING || strcmp(vals, "_openxlsx_NA") == 0) {
-
         if (na_missing) {
           // v = "#N/A"
           SET_STRING_ELT(zz_v,   pos, na_sexp);
@@ -555,7 +561,7 @@ void wide_to_long(
       }
 
       // typ = std::to_string(vtyp)
-      SET_STRING_ELT(zz_typ, pos, Rf_mkChar(std::to_string(vtyp).c_str()));
+      if (has_typ) SET_STRING_ELT(zz_typ, pos, Rf_mkChar(std::to_string(vtyp).c_str()));
 
       std::string cell_r = has_dims ? dims[static_cast<size_t>(idx)] : col + row;
       SET_STRING_ELT(zz_r, pos, Rf_mkChar(cell_r.c_str()));
@@ -569,8 +575,8 @@ void wide_to_long(
         SET_STRING_ELT(zz_row_r, pos, Rf_mkChar(row.c_str()));
         SET_STRING_ELT(zz_c_r, pos, Rf_mkChar(col.c_str()));
       }
-    } // n
-  } // m
+    }  // n
+  }  // m
 }
 
 // simple helper function to create a data frame of type character
@@ -579,7 +585,6 @@ void wide_to_long(
 //' @noRd
 // [[Rcpp::export]]
 Rcpp::DataFrame create_char_dataframe(Rcpp::CharacterVector colnames, R_xlen_t n) {
-
   R_xlen_t kk = static_cast<R_xlen_t>(colnames.size());
 
   // 1. create the list
@@ -602,7 +607,6 @@ Rcpp::DataFrame create_char_dataframe(Rcpp::CharacterVector colnames, R_xlen_t n
 
 // [[Rcpp::export]]
 Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::string> vec_attrs, std::vector<std::string> vec_chlds) {
-
   std::set<std::string> nam_attrs(vec_attrs.begin(), vec_attrs.end());
   std::set<std::string> nam_chlds(vec_chlds.begin(), vec_chlds.end());
 
@@ -615,7 +619,6 @@ Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::
   std::set<std::string> nams(std::make_move_iterator(all_names.begin()),
                              std::make_move_iterator(all_names.end()));
 
-
   R_xlen_t nn = std::distance(xml->begin(), xml->end());
   R_xlen_t kk = static_cast<R_xlen_t>(nams.size());
   uint32_t pugi_format_flags = pugi::format_raw | pugi::format_no_escapes;
@@ -624,8 +627,7 @@ Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::
 
   // 1. create the list
   Rcpp::List df(kk);
-  for (R_xlen_t i = 0; i < kk; ++i)
-  {
+  for (R_xlen_t i = 0; i < kk; ++i) {
     SET_VECTOR_ELT(df, i, Rcpp::CharacterVector(Rcpp::no_init(nn)));
   }
 
@@ -634,7 +636,6 @@ Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::
   auto itr = 0;
   for (auto xml_node : xml->children(vec_name.c_str())) {
     for (auto attrs : xml_node.attributes()) {
-
       std::string attr_name = attrs.name();
       std::string attr_value = attrs.value();
       auto find_res = nams.find(attr_name);
@@ -649,7 +650,6 @@ Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::
     }
 
     for (auto cld : xml_node.children()) {
-
       std::string cld_name = cld.name();
       auto find_res = nams.find(cld_name);
 
@@ -668,7 +668,6 @@ Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::
 
     rvec[itr] = std::to_string(itr);
     ++itr;
-
   }
 
   // 3. Create a data.frame
@@ -679,17 +678,15 @@ Rcpp::DataFrame read_xml2df(XPtrXML xml, std::string vec_name, std::vector<std::
   return df;
 }
 
-
 // [[Rcpp::export]]
 Rcpp::CharacterVector write_df2xml(Rcpp::DataFrame df, std::string vec_name, std::vector<std::string> vec_attrs, std::vector<std::string> vec_chlds) {
-
   int64_t n = df.nrow();
   Rcpp::CharacterVector z(n);
   uint32_t pugi_parse_flags = pugi::parse_cdata | pugi::parse_wconv_attribute | pugi::parse_ws_pcdata | pugi::parse_eol;
   uint32_t pugi_format_flags = pugi::format_raw | pugi::format_no_escapes;
 
   // openxml 2.8.1
-  std::vector<std::string>  attrnams = df.names();
+  std::vector<std::string> attrnams = df.names();
   std::set<std::string> nam_attrs(vec_attrs.begin(), vec_attrs.end());
   std::set<std::string> nam_chlds(vec_chlds.begin(), vec_chlds.end());
 
@@ -698,7 +695,6 @@ Rcpp::CharacterVector write_df2xml(Rcpp::DataFrame df, std::string vec_name, std
     pugi::xml_node xml_node = doc.append_child(vec_name.c_str());
 
     for (auto j = 0; j < df.ncol(); ++j) {
-
       std::string attr_j = attrnams[static_cast<size_t>(j)];
 
       // mimic which
@@ -732,7 +728,6 @@ Rcpp::CharacterVector write_df2xml(Rcpp::DataFrame df, std::string vec_name, std
         cv_s = Rcpp::as<Rcpp::CharacterVector>(df[j])[i];
 
         if (cv_s[0] != "") {
-
           std::string child_i = Rcpp::as<std::string>(cv_s[0]);
 
           pugi::xml_document xml_child;
@@ -740,7 +735,6 @@ Rcpp::CharacterVector write_df2xml(Rcpp::DataFrame df, std::string vec_name, std
           if (!result) Rcpp::stop("loading %s child node fail: %s", vec_name, cv_s);
 
           xml_node.append_copy(xml_child.first_child());
-
         }
       }
 
