@@ -1,9 +1,8 @@
 #include "openxlsx2.h"
 
 // [[Rcpp::export]]
-SEXP readXMLPtr(std::string path, bool isfile, bool escapes, bool declaration, bool whitespace, bool empty_tags, bool skip_control) {
-
-  xmldoc *doc = new xmldoc;
+SEXP readXML(std::string path, bool isfile, bool escapes, bool declaration, bool whitespace, bool empty_tags, bool skip_control, bool pointer) {
+  xmldoc* doc = new xmldoc;
   pugi::xml_parse_result result;
 
   // pugi::parse_default without escapes flag
@@ -23,48 +22,26 @@ SEXP readXMLPtr(std::string path, bool isfile, bool escapes, bool declaration, b
     Rcpp::stop("xml import unsuccessful");
   }
 
-  XPtrXML ptr(doc, true);
-  ptr.attr("class") = Rcpp::CharacterVector::create("pugi_xml");
-  ptr.attr("escapes") = escapes;
-  ptr.attr("empty_tags") = empty_tags;
-  ptr.attr("skip_control") = skip_control;
-  return ptr;
-}
-
-// [[Rcpp::export]]
-SEXP readXML(std::string path, bool isfile, bool escapes, bool declaration, bool whitespace, bool empty_tags, bool skip_control) {
-
-  pugi::xml_document doc;
-  pugi::xml_parse_result result;
-
-  // pugi::parse_default without escapes flag
-  uint32_t pugi_parse_flags = pugi::parse_cdata | pugi::parse_wconv_attribute | pugi::parse_eol;
-  if (escapes) pugi_parse_flags |= pugi::parse_escapes;
-  if (declaration) pugi_parse_flags |= pugi::parse_declaration;
-  if (whitespace) pugi_parse_flags |= pugi::parse_ws_pcdata_single;
-  if (!whitespace) pugi_parse_flags |= pugi::parse_trim_pcdata;
-
   uint32_t pugi_format_flags = pugi::format_raw;
   if (!escapes) pugi_format_flags |= pugi::format_no_escapes;
   if (empty_tags) pugi_format_flags |= pugi::format_no_empty_element_tags;
   if (skip_control) pugi_format_flags |= pugi::format_skip_control_chars;
 
-  if (isfile) {
-    result = doc.load_file(path.c_str(), pugi_parse_flags, pugi::encoding_utf8);
-  } else {
-    result = doc.load_string(path.c_str(), pugi_parse_flags);
-  }
-
-  if (!result) {
-    Rcpp::stop("xml import unsuccessful");
+  if (pointer) {
+    XPtrXML ptr(doc, true);
+    ptr.attr("class") = Rcpp::CharacterVector::create("pugi_xml");
+    ptr.attr("escapes") = escapes;
+    ptr.attr("empty_tags") = empty_tags;
+    ptr.attr("skip_control") = skip_control;
+    return ptr;
   }
 
   std::ostringstream oss;
-  doc.print(oss, " ", pugi_format_flags);
-  return  Rcpp::wrap(Rcpp::String(oss.str()));
+  doc->print(oss, " ", pugi_format_flags);
+  return Rcpp::wrap(Rcpp::String(oss.str()));
 }
 
-inline uint32_t pugi_format(XPtrXML doc){
+inline uint32_t pugi_format(XPtrXML doc) {
   bool escapes = Rcpp::as<bool>(doc.attr("escapes"));
   bool empty_tags = Rcpp::as<bool>(doc.attr("empty_tags"));
   bool skip_control = Rcpp::as<bool>(doc.attr("skip_control"));
@@ -79,7 +56,6 @@ inline uint32_t pugi_format(XPtrXML doc){
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector is_xml(std::string str) {
-
   pugi::xml_document doc;
   pugi::xml_parse_result result;
 
@@ -93,90 +69,75 @@ Rcpp::LogicalVector is_xml(std::string str) {
 
 // [[Rcpp::export]]
 SEXP getXMLXPtrName1(XPtrXML doc) {
-
   vec_string res;
 
-  for (auto lvl0 : doc->children())
-  {
+  for (auto lvl0 : doc->children()) {
     res.push_back(lvl0.name());
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtrName2(XPtrXML doc, std::string level1) {
-
   vec_string res;
 
-  for (auto lvl0 : doc->children(level1.c_str()))
-  {
-    for (auto lvl1 : lvl0.children())
-    {
+  for (auto lvl0 : doc->children(level1.c_str())) {
+    for (auto lvl1 : lvl0.children()) {
       res.push_back(lvl1.name());
     }
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtrName3(XPtrXML doc, std::string level1, std::string level2) {
-
   vec_string res;
 
-  for (auto lvl0 : doc->children(level1.c_str()))
-  {
-    for (auto lvl1 : lvl0.children())
-    {
-      for (auto lvl2 : lvl1.children())
-      {
+  for (auto lvl0 : doc->children(level1.c_str())) {
+    for (auto lvl1 : lvl0.children()) {
+      for (auto lvl2 : lvl1.children()) {
         res.push_back(lvl2.name());
       }
     }
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr0(XPtrXML doc) {
-
   vec_string res;
-  uint32_t  pugi_format_flags = pugi_format(doc);
+  uint32_t pugi_format_flags = pugi_format(doc);
 
-  for (auto worksheet : doc->children())
-  {
+  for (auto worksheet : doc->children()) {
     std::ostringstream oss;
     worksheet.print(oss, " ", pugi_format_flags);
     res.push_back(Rcpp::String(oss.str()));
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr1(XPtrXML doc, std::string child) {
-
   vec_string res;
-  uint32_t  pugi_format_flags = pugi_format(doc);
+  uint32_t pugi_format_flags = pugi_format(doc);
 
-  for (auto cld : doc->children(child.c_str()))
-  {
+  for (auto cld : doc->children(child.c_str())) {
     std::ostringstream oss;
     cld.print(oss, " ", pugi_format_flags);
     res.push_back(Rcpp::String(oss.str()));
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
-
 // [[Rcpp::export]]
 SEXP getXMLXPtr2(XPtrXML doc, std::string level1, std::string child) {
-
   vec_string res;
-  uint32_t  pugi_format_flags = pugi_format(doc);
+  uint32_t pugi_format_flags = pugi_format(doc);
 
   for (auto lvl1 : doc->children(level1.c_str())) {
     for (auto cld : lvl1.children(child.c_str())) {
@@ -186,14 +147,13 @@ SEXP getXMLXPtr2(XPtrXML doc, std::string level1, std::string child) {
     }
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr3(XPtrXML doc, std::string level1, std::string level2, std::string child) {
-
   vec_string res;
-  uint32_t  pugi_format_flags = pugi_format(doc);
+  uint32_t pugi_format_flags = pugi_format(doc);
 
   for (auto lvl1 : doc->children(level1.c_str())) {
     for (auto lvl2 : lvl1.children(level2.c_str())) {
@@ -205,16 +165,15 @@ SEXP getXMLXPtr3(XPtrXML doc, std::string level1, std::string level2, std::strin
     }
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // level2 is wildcard. (for border only color nodes are imported.
 // Do not know why :'( )
 // [[Rcpp::export]]
 SEXP unkgetXMLXPtr3(XPtrXML doc, std::string level1, std::string child) {
-
   vec_string res;
-  uint32_t  pugi_format_flags = pugi_format(doc);
+  uint32_t pugi_format_flags = pugi_format(doc);
 
   for (auto lvl1 : doc->children(level1.c_str())) {
     for (auto lvl2 : lvl1.children()) {
@@ -226,17 +185,15 @@ SEXP unkgetXMLXPtr3(XPtrXML doc, std::string level1, std::string child) {
     }
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLPtr1con(XPtrXML doc) {
-
   vec_string res;
-  uint32_t  pugi_format_flags = pugi_format(doc);
+  uint32_t pugi_format_flags = pugi_format(doc);
 
-  for (auto node : doc->children())
-  {
+  for (auto node : doc->children()) {
     for (auto cld : node.children()) {
       std::ostringstream oss;
       cld.print(oss, " ", pugi_format_flags);
@@ -244,63 +201,53 @@ SEXP getXMLPtr1con(XPtrXML doc) {
     }
   }
 
-  return  Rcpp::wrap(res);
+  return Rcpp::wrap(res);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr1val(XPtrXML doc, std::string child) {
-
   // returns a single vector, not a list of vectors!
   std::vector<std::string> x;
 
-  for (auto worksheet : doc->children(child.c_str()))
-  {
+  for (auto worksheet : doc->children(child.c_str())) {
     x.push_back(Rcpp::String(worksheet.text().get()));
   }
 
-  return  Rcpp::wrap(x);
+  return Rcpp::wrap(x);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr2val(XPtrXML doc, std::string level1, std::string child) {
-
   // returns a single vector, not a list of vectors!
   std::vector<std::string> x;
 
   for (auto worksheet : doc->children(level1.c_str())) {
-
     for (auto col : worksheet.children(child.c_str())) {
       x.push_back(Rcpp::String(col.text().get()));
     }
   }
 
-  return  Rcpp::wrap(x);
+  return Rcpp::wrap(x);
 }
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr3val(XPtrXML doc, std::string level1, std::string level2, std::string child) {
-
   // returns a single vector, not a list of vectors!
   std::vector<std::string> x;
 
-  for (auto worksheet : doc->child(level1.c_str()).children(level2.c_str()))
-  {
-
+  for (auto worksheet : doc->child(level1.c_str()).children(level2.c_str())) {
     for (auto col : worksheet.children(child.c_str()))
       x.push_back(Rcpp::String(col.text().get()));
   }
 
-  return  Rcpp::wrap(x);
+  return Rcpp::wrap(x);
 }
 
-
 // [[Rcpp::export]]
 SEXP getXMLXPtr1attr(XPtrXML doc, std::string child) {
-
   auto children = doc->children(child.c_str());
 
-  R_xlen_t n = std::distance(children.begin(),
-                             children.end());
+  R_xlen_t n = std::distance(children.begin(), children.end());
 
   // for a childless single line node the distance might be zero
   if (n == 0) n++;
@@ -308,12 +255,10 @@ SEXP getXMLXPtr1attr(XPtrXML doc, std::string child) {
 
   auto itr = 0;
   for (auto chld : children) {
-
     Rcpp::CharacterVector res;
     std::vector<std::string> nam;
 
-    for (auto attrs : chld.attributes())
-    {
+    for (auto attrs : chld.attributes()) {
       nam.push_back(Rcpp::String(attrs.name()));
       res.push_back(Rcpp::String(attrs.value()));
     }
@@ -330,23 +275,19 @@ SEXP getXMLXPtr1attr(XPtrXML doc, std::string child) {
 
 // [[Rcpp::export]]
 Rcpp::List getXMLXPtr2attr(XPtrXML doc, std::string level1, std::string child) {
-
   auto worksheet = doc->child(level1.c_str()).children(child.c_str());
-  R_xlen_t n = std::distance(worksheet.begin() , worksheet.end());
+  R_xlen_t n = std::distance(worksheet.begin(), worksheet.end());
   Rcpp::List z(n);
 
   auto itr = 0;
-  for (auto ws : worksheet)
-  {
-
+  for (auto ws : worksheet) {
     R_xlen_t w_n = std::distance(ws.attributes_begin(), ws.attributes_end());
 
     Rcpp::CharacterVector res(w_n);
     Rcpp::CharacterVector nam(w_n);
 
     auto attr_itr = 0;
-    for (auto attr : ws.attributes())
-    {
+    for (auto attr : ws.attributes()) {
       nam[attr_itr] = Rcpp::String(attr.name());
       res[attr_itr] = Rcpp::String(attr.value());
       ++attr_itr;
@@ -364,23 +305,19 @@ Rcpp::List getXMLXPtr2attr(XPtrXML doc, std::string level1, std::string child) {
 
 // [[Rcpp::export]]
 SEXP getXMLXPtr3attr(XPtrXML doc, std::string level1, std::string level2, std::string child) {
-
   auto worksheet = doc->child(level1.c_str()).child(level2.c_str()).children(child.c_str());
   R_xlen_t n = std::distance(worksheet.begin(), worksheet.end());
   Rcpp::List z(n);
 
   auto itr = 0;
-  for (auto ws : worksheet)
-  {
-
+  for (auto ws : worksheet) {
     R_xlen_t w_n = std::distance(ws.attributes_begin(), ws.attributes_end());
 
     Rcpp::CharacterVector res(w_n);
     Rcpp::CharacterVector nam(w_n);
 
     auto attr_itr = 0;
-    for (auto attr : ws.attributes())
-    {
+    for (auto attr : ws.attributes()) {
       nam[attr_itr] = Rcpp::String(attr.name());
       res[attr_itr] = Rcpp::String(attr.value());
       ++attr_itr;
@@ -396,10 +333,8 @@ SEXP getXMLXPtr3attr(XPtrXML doc, std::string level1, std::string level2, std::s
   return z;
 }
 
-
 // [[Rcpp::export]]
 SEXP printXPtr(XPtrXML doc, std::string indent, bool raw, bool attr_indent) {
-
   // pugi::parse_default without escapes flag
   uint32_t pugi_format_flags = pugi_format(doc);
   if (!raw) {
@@ -415,11 +350,9 @@ SEXP printXPtr(XPtrXML doc, std::string indent, bool raw, bool attr_indent) {
   return Rcpp::wrap(Rcpp::String(oss.str()));
 }
 
-
 // [[Rcpp::export]]
 XPtrXML write_xml_file(std::string xml_content, bool escapes) {
-
-  xmldoc *doc = new xmldoc;
+  xmldoc* doc = new xmldoc;
 
   // pugi::parse_default without escapes flag
   uint32_t pugi_parse_flags = pugi::parse_cdata | pugi::parse_wconv_attribute | pugi::parse_ws_pcdata | pugi::parse_eol;
@@ -480,10 +413,11 @@ XPtrXML write_xml_file(std::string xml_content, bool escapes) {
 //'     xml_attr_mod(xml_node, xml_attr)
 //' @export
 // [[Rcpp::export]]
-Rcpp::CharacterVector xml_attr_mod(std::string xml_content, Rcpp::CharacterVector xml_attributes,
-                                   bool escapes = false, bool declaration = false,
+Rcpp::CharacterVector xml_attr_mod(std::string xml_content,
+                                   Rcpp::CharacterVector xml_attributes,
+                                   bool escapes = false,
+                                   bool declaration = false,
                                    bool remove_empty_attr = true) {
-
   pugi::xml_document doc;
   pugi::xml_parse_result result;
 
@@ -504,8 +438,7 @@ Rcpp::CharacterVector xml_attr_mod(std::string xml_content, Rcpp::CharacterVecto
   std::vector<std::string> new_attr_val = Rcpp::as<std::vector<std::string>>(xml_attributes);
 
   for (auto cld : doc.children()) {
-    for (size_t i = 0; i < static_cast<size_t>(xml_attributes.length()); ++i){
-
+    for (size_t i = 0; i < static_cast<size_t>(xml_attributes.length()); ++i) {
       // check if attribute_val is empty. if yes, remove the attribute.
       // otherwise add or update the attribute
       if (new_attr_val[i].empty()) {
@@ -560,7 +493,6 @@ Rcpp::CharacterVector xml_node_create(
     Rcpp::Nullable<Rcpp::CharacterVector> xml_children = R_NilValue,
     Rcpp::Nullable<Rcpp::CharacterVector> xml_attributes = R_NilValue,
     bool escapes = false, bool declaration = false) {
-
   pugi::xml_document doc;
 
   uint32_t pugi_parse_flags = pugi::parse_cdata | pugi::parse_wconv_attribute | pugi::parse_ws_pcdata | pugi::parse_eol;
@@ -574,11 +506,9 @@ Rcpp::CharacterVector xml_node_create(
 
   // check if children are attached
   if (xml_children.isNotNull()) {
-
     Rcpp::CharacterVector xml_child(xml_children.get());
 
     for (auto i = 0; i < xml_child.size(); ++i) {
-
       std::string xml_cld = std::string(xml_child[i]);
 
       pugi::xml_document is_node;
@@ -596,13 +526,12 @@ Rcpp::CharacterVector xml_node_create(
 
   // check if attributes are attached
   if (xml_attributes.isNotNull()) {
-
     Rcpp::CharacterVector xml_attr(xml_attributes.get());
 
     std::vector<std::string> new_attr_nam = xml_attr.names();
     std::vector<std::string> new_attr_val = Rcpp::as<std::vector<std::string>>(xml_attr);
 
-    for (size_t i = 0; i < static_cast<size_t>(xml_attr.length()); ++i){
+    for (size_t i = 0; i < static_cast<size_t>(xml_attr.length()); ++i) {
       if (!new_attr_val[i].empty())
         cld.append_attribute(new_attr_nam[i].c_str()) = new_attr_val[i].c_str();
     }
@@ -621,10 +550,9 @@ Rcpp::CharacterVector xml_node_create(
 // @export
 // [[Rcpp::export]]
 SEXP xml_append_child1(XPtrXML node, XPtrXML child, bool pointer) {
-
   uint32_t pugi_format_flags = pugi_format(node);
 
-  for (auto cld: child->children()) {
+  for (auto cld : child->children()) {
     node->first_child().append_copy(cld);
   }
 
@@ -645,10 +573,9 @@ SEXP xml_append_child1(XPtrXML node, XPtrXML child, bool pointer) {
 // @export
 // [[Rcpp::export]]
 SEXP xml_append_child2(XPtrXML node, XPtrXML child, std::string level1, bool pointer) {
-
   uint32_t pugi_format_flags = pugi_format(node);
 
-  for (auto cld: child->children()) {
+  for (auto cld : child->children()) {
     node->first_child().child(level1.c_str()).append_copy(cld);
   }
 
@@ -670,10 +597,9 @@ SEXP xml_append_child2(XPtrXML node, XPtrXML child, std::string level1, bool poi
 // @export
 // [[Rcpp::export]]
 SEXP xml_append_child3(XPtrXML node, XPtrXML child, std::string level1, std::string level2, bool pointer) {
-
   uint32_t pugi_format_flags = pugi_format(node);
 
-  for (auto cld: child->children()) {
+  for (auto cld : child->children()) {
     node->first_child().child(level1.c_str()).child(level2.c_str()).append_copy(cld);
   }
 
@@ -694,14 +620,14 @@ SEXP xml_append_child3(XPtrXML node, XPtrXML child, std::string level1, std::str
 // @export
 // [[Rcpp::export]]
 SEXP xml_remove_child1(XPtrXML node, std::string child, int32_t which, bool pointer) {
-
   uint32_t pugi_format_flags = pugi_format(node);
 
   auto ctr = 0;
-  for (pugi::xml_node cld = node->first_child().child(child.c_str()); cld; ) {
+  for (pugi::xml_node cld = node->first_child().child(child.c_str()); cld;) {
     auto next = cld.next_sibling();
-    if (ctr == which || which < 0) cld.parent().remove_child(cld);
-     cld = next;
+    if (ctr == which || which < 0)
+      cld.parent().remove_child(cld);
+    cld = next;
     ++ctr;
   }
 
@@ -722,14 +648,14 @@ SEXP xml_remove_child1(XPtrXML node, std::string child, int32_t which, bool poin
 // @param escapes bool if escapes should be used
 // @export
 // [[Rcpp::export]]
-SEXP xml_remove_child2(XPtrXML node, std::string child, std::string level1, int32_t which,  bool pointer) {
-
+SEXP xml_remove_child2(XPtrXML node, std::string child, std::string level1, int32_t which, bool pointer) {
   uint32_t pugi_format_flags = pugi_format(node);
 
   auto ctr = 0;
-  for (pugi::xml_node cld = node->first_child().child(level1.c_str()).child(child.c_str()); cld; ) {
+  for (pugi::xml_node cld = node->first_child().child(level1.c_str()).child(child.c_str()); cld;) {
     auto next = cld.next_sibling();
-    if (ctr == which || which < 0)  cld.parent().remove_child(cld);
+    if (ctr == which || which < 0)
+      cld.parent().remove_child(cld);
     cld = next;
     ++ctr;
   }
@@ -753,13 +679,13 @@ SEXP xml_remove_child2(XPtrXML node, std::string child, std::string level1, int3
 // @export
 // [[Rcpp::export]]
 SEXP xml_remove_child3(XPtrXML node, std::string child, std::string level1, std::string level2, int32_t which, bool pointer) {
-
   uint32_t pugi_format_flags = pugi_format(node);
 
   auto ctr = 0;
-  for (pugi::xml_node cld = node->first_child().child(level1.c_str()).child(level2.c_str()).child(child.c_str()); cld; ) {
+  for (pugi::xml_node cld = node->first_child().child(level1.c_str()).child(level2.c_str()).child(child.c_str()); cld;) {
     auto next = cld.next_sibling();
-    if (ctr == which || which < 0) cld.parent().remove_child(cld);
+    if (ctr == which || which < 0)
+      cld.parent().remove_child(cld);
     cld = next;
     ++ctr;
   }
