@@ -1,9 +1,8 @@
-#include "openxlsx2.h"
 #include <fstream>
+#include "openxlsx2.h"
 
 // [[Rcpp::export]]
 Rcpp::CharacterVector set_sst(Rcpp::CharacterVector sharedStrings) {
-
   Rcpp::CharacterVector sst(sharedStrings.length());
 
   for (auto i = 0; i < sharedStrings.length(); ++i) {
@@ -21,20 +20,18 @@ Rcpp::CharacterVector set_sst(Rcpp::CharacterVector sharedStrings) {
   return sst;
 }
 
-
 // creates an xml row
 // data in xml is ordered row wise. therefore we need the row attributes and
 // the column data used in this row. This function uses both to create a single
 // row and passes it to write_worksheet_xml_2 which will create the entire
 // sheet_data part for this worksheet
-pugi::xml_document xml_sheet_data(Rcpp::DataFrame row_attr, Rcpp::DataFrame cc) {
 
-  auto lastrow = 0; // integer value of the last row with column data
-  auto thisrow = 0; // integer value of the current row with column data
-  auto row_idx = 0; // the index of the row_attr file. this is != rowid
-  auto rowid   = 0; // integer value of the r field in row_attr
+void xml_sheet_data(pugi::xml_node &doc, Rcpp::DataFrame &row_attr, Rcpp::DataFrame &cc) {
+  auto lastrow = 0;  // integer value of the last row with column data
+  auto thisrow = 0;  // integer value of the current row with column data
+  auto row_idx = 0;  // the index of the row_attr file. this is != rowid
+  auto rowid   = 0;  // integer value of the r field in row_attr
 
-  pugi::xml_document doc;
   pugi::xml_node row;
 
   std::string xml_preserver = " ";
@@ -43,45 +40,42 @@ pugi::xml_document xml_sheet_data(Rcpp::DataFrame row_attr, Rcpp::DataFrame cc)
   uint32_t pugi_parse_flags = pugi::parse_cdata | pugi::parse_wconv_attribute | pugi::parse_ws_pcdata | pugi::parse_eol;
   // uint32_t pugi_format_flags = pugi::format_raw | pugi::format_no_escapes;
 
+  bool has_cm = cc.containsElementNamed("c_cm");
+  bool has_ph = cc.containsElementNamed("c_ph");
+  bool has_vm = cc.containsElementNamed("c_vm");
+
+  Rcpp::CharacterVector cc_c_cm, cc_c_ph, cc_c_vm;
+
   // we cannot access rows directly in the dataframe.
   // Have to extract the columns and use these
-  Rcpp::CharacterVector cc_row_r = cc["row_r"]; // 1
-  Rcpp::CharacterVector cc_r     = cc["r"];     // A1
-  Rcpp::CharacterVector cc_v     = cc["v"];
-  Rcpp::CharacterVector cc_c_t   = cc["c_t"];
-  Rcpp::CharacterVector cc_c_s   = cc["c_s"];
-  Rcpp::CharacterVector cc_c_cm  = cc["c_cm"];
-  Rcpp::CharacterVector cc_c_ph  = cc["c_ph"];
-  Rcpp::CharacterVector cc_c_vm  = cc["c_vm"];
-  Rcpp::CharacterVector cc_f     = cc["f"];
-  Rcpp::CharacterVector cc_f_t   = cc["f_t"];
-  Rcpp::CharacterVector cc_f_ref = cc["f_ref"];
-  Rcpp::CharacterVector cc_f_ca  = cc["f_ca"];
-  Rcpp::CharacterVector cc_f_si  = cc["f_si"];
-  Rcpp::CharacterVector cc_is    = cc["is"];
+  Rcpp::CharacterVector cc_row_r  = cc["row_r"];  // 1
+  Rcpp::CharacterVector cc_r      = cc["r"];      // A1
+  Rcpp::CharacterVector cc_v      = cc["v"];      // can be utf8
+  Rcpp::CharacterVector cc_c_t    = cc["c_t"];
+  Rcpp::CharacterVector cc_c_s    = cc["c_s"];
+  if (has_cm) cc_c_cm   = cc["c_cm"];
+  if (has_ph) cc_c_ph   = cc["c_ph"];
+  if (has_vm) cc_c_vm   = cc["c_vm"];
+  Rcpp::CharacterVector cc_f      = cc["f"];      // can be utf8
+  Rcpp::CharacterVector cc_f_attr = cc["f_attr"];
+  Rcpp::CharacterVector cc_is     = cc["is"];     // can be utf8
 
   Rcpp::CharacterVector row_r    = row_attr["r"];
 
   for (auto i = 0; i < cc.nrow(); ++i) {
-
     thisrow = std::stoi(Rcpp::as<std::string>(cc_row_r[i]));
 
     if (lastrow < thisrow) {
-
       // there might be entirely empty rows in between. this is the case for
       // loadExample. We check the rowid and write the line and skip until we
       // have every row and only then continue writing the column
       while (rowid < thisrow) {
-
-        rowid = std::stoi(Rcpp::as<std::string>(
-          row_r[row_idx]
-        ));
+        rowid = std::stoi(Rcpp::as<std::string>(row_r[row_idx]));
 
         row = doc.append_child("row");
         Rcpp::CharacterVector attrnams = row_attr.names();
 
         for (auto j = 0; j < row_attr.ncol(); ++j) {
-
           Rcpp::CharacterVector cv_s = "";
           cv_s = Rcpp::as<Rcpp::CharacterVector>(row_attr[j])[row_idx];
 
@@ -99,19 +93,16 @@ pugi::xml_document xml_sheet_data(Rcpp::DataFrame row_attr, Rcpp::DataFrame cc)
     // update lastrow
     lastrow = thisrow;
 
-    if ( // skip blank cells entirely
-        cc_c_s[i]   == "" &&
-        cc_c_t[i]   == "" &&
-        cc_c_cm[i]  == "" &&
-        cc_c_ph[i]  == "" &&
-        cc_c_vm[i]  == "" &&
-        cc_v[i]     == "" &&
-        cc_f[i]     == "" &&
-        cc_f_t[i]   == "" &&
-        cc_f_ref[i] == "" &&
-        cc_f_ca[i]  == "" &&
-        cc_f_si[i]  == "" &&
-        cc_is[i]    == ""
+    if (
+      cc_c_s[i].empty() &&
+      cc_c_t[i].empty() &&
+      (!has_cm || (has_cm && cc_c_cm[i].empty())) &&
+      (!has_ph || (has_ph && cc_c_ph[i].empty())) &&
+      (!has_vm || (has_vm && cc_c_vm[i].empty())) &&
+      cc_v[i].empty() &&
+      cc_f[i].empty() &&
+      cc_f_attr[i].empty() &&
+      cc_is[i].empty()
     ) {
       continue;
     }
@@ -123,26 +114,26 @@ pugi::xml_document xml_sheet_data(Rcpp::DataFrame row_attr, Rcpp::DataFrame cc)
     // additional attr list.
 
     // append attributes <c r="A1" ...>
-    cell.append_attribute("r") = to_string(cc_r[i]).c_str();
+    cell.append_attribute("r") = std::string(cc_r[i]).c_str();
 
-    if (!to_string(cc_c_s[i]).empty())
-      cell.append_attribute("s") = to_string(cc_c_s[i]).c_str();
+    if (!std::string(cc_c_s[i]).empty())
+      cell.append_attribute("s") = std::string(cc_c_s[i]).c_str();
 
     // assign type if not <v> aka numeric
-    if (!to_string(cc_c_t[i]).empty())
-      cell.append_attribute("t") = to_string(cc_c_t[i]).c_str();
+    if (!std::string(cc_c_t[i]).empty())
+      cell.append_attribute("t") = std::string(cc_c_t[i]).c_str();
 
     // CellMetaIndex: suppress curly brackets in spreadsheet software
-    if (!to_string(cc_c_cm[i]).empty())
-      cell.append_attribute("cm") = to_string(cc_c_cm[i]).c_str();
+    if (has_cm && !std::string(cc_c_cm[i]).empty())
+      cell.append_attribute("cm") = std::string(cc_c_cm[i]).c_str();
 
     // phonetics spelling
-    if (!to_string(cc_c_ph[i]).empty())
-      cell.append_attribute("ph") = to_string(cc_c_ph[i]).c_str();
+    if (has_ph && !std::string(cc_c_ph[i]).empty())
+      cell.append_attribute("ph") = std::string(cc_c_ph[i]).c_str();
 
     // suppress curly brackets in spreadsheet software
-    if (!to_string(cc_c_vm[i]).empty())
-      cell.append_attribute("vm") = to_string(cc_c_vm[i]).c_str();
+    if (has_vm && !std::string(cc_c_vm[i]).empty())
+      cell.append_attribute("vm") = std::string(cc_c_vm[i]).c_str();
 
     // append nodes <c r="A1" ...><v>...</v></c>
 
@@ -150,39 +141,51 @@ pugi::xml_document xml_sheet_data(Rcpp::DataFrame row_attr, Rcpp::DataFrame cc)
 
     // <f> ... </f>
     // f node: formula to be evaluated
-    if (!to_string(cc_f[i]).empty() || !to_string(cc_f_t[i]).empty() || !to_string(cc_f_si[i]).empty()) {
+    if (!std::string(cc_f[i]).empty() || !std::string(cc_f_attr[i]).empty()) {
+      // Fix Most Vexing Parse
+      std::istringstream attr_stream((std::string(cc_f_attr[i])));
       pugi::xml_node f = cell.append_child("f");
-      if (!to_string(cc_f_t[i]).empty()) {
-        f.append_attribute("t") = to_string(cc_f_t[i]).c_str();
-      }
-      if (!to_string(cc_f_ref[i]).empty()) {
-        f.append_attribute("ref") = to_string(cc_f_ref[i]).c_str();
-      }
-      if (!to_string(cc_f_ca[i]).empty()) {
-        f.append_attribute("ca") = to_string(cc_f_ca[i]).c_str();
-      }
-      if (!to_string(cc_f_si[i]).empty()) {
-        f.append_attribute("si") = to_string(cc_f_si[i]).c_str();
-        f_si = true;
+
+      // Parse attributes from f_attr
+      std::string key_value;
+      while (std::getline(attr_stream, key_value, ' ')) {
+          auto pos = key_value.find('=');
+          if (pos != std::string::npos) {
+              std::string key = key_value.substr(0, pos);
+              std::string value = key_value.substr(pos + 1);
+
+              // Remove quotes from value
+              if (!value.empty() && value.front() == '\"' && value.back() == '\"') {
+                  value = value.substr(1, value.size() - 2);
+              }
+
+              // Append attribute to <f>
+              f.append_attribute(key.c_str()) = value.c_str();
+          }
       }
 
-      f.append_child(pugi::node_pcdata).set_value(to_string(cc_f[i]).c_str());
+      // Add the content of <f>
+      if (!std::string(cc_f[i]).empty()) {
+          f.append_child(pugi::node_pcdata).set_value(std::string(cc_f[i]).c_str());
+      }
     }
 
     // v node: value stored from evaluated formula
-    if (!to_string(cc_v[i]).empty()) {
+    if (!std::string(cc_v[i]).empty()) {
       if (!f_si & (to_string(cc_v[i]).compare(xml_preserver.c_str()) == 0)) {
         cell.append_child("v").append_attribute("xml:space").set_value("preserve");
         cell.child("v").append_child(pugi::node_pcdata).set_value(" ");
       } else {
-        cell.append_child("v").append_child(pugi::node_pcdata).set_value(to_string(cc_v[i]).c_str());
+        if (std::string(cc_c_t[i]).empty() && std::string(cc_f_attr[i]).empty())
+          cell.append_child("v").append_child(pugi::node_pcdata).set_value(std::string(cc_v[i]).c_str());
+        else
+          cell.append_child("v").append_child(pugi::node_pcdata).set_value(to_string(cc_v[i]).c_str());
       }
     }
 
     // <is><t> ... </t></is>
-    if (to_string(cc_c_t[i]).compare("inlineStr") == 0) {
-      if (!to_string(cc_is[i]).empty()) {
-
+    if (std::string(cc_c_t[i]).compare("inlineStr") == 0) {
+      if (!std::string(cc_is[i]).empty()) {
         pugi::xml_document is_node;
         pugi::xml_parse_result result = is_node.load_string(to_string(cc_is[i]).c_str(), pugi_parse_flags);
         if (!result) Rcpp::stop("loading inlineStr node while writing failed");
@@ -191,33 +194,23 @@ pugi::xml_document xml_sheet_data(Rcpp::DataFrame row_attr, Rcpp::DataFrame cc)
       }
     }
   }
-
-  return doc;
 }
 
-
 // TODO: convert to pugi
 // function that creates the xml worksheet
 // uses preparated data and writes it. It passes data to set_row() which will
 // create single xml rows of sheet_data.
 //
 // [[Rcpp::export]]
-XPtrXML write_worksheet(
-    std::string prior,
-    std::string post,
-    Rcpp::Environment sheet_data
-) {
-
+XPtrXML write_worksheet(std::string prior, std::string post, Rcpp::Environment sheet_data) {
   uint32_t pugi_parse_flags = pugi::parse_cdata | pugi::parse_wconv_attribute | pugi::parse_ws_pcdata | pugi::parse_eol;
 
-
   // sheet_data will be in order, just need to check for row_heights
   // CharacterVector cell_col = int_to_col(sheet_data.field("cols"));
   Rcpp::DataFrame row_attr = Rcpp::as<Rcpp::DataFrame>(sheet_data["row_attr"]);
-  Rcpp::DataFrame cc = Rcpp::as<Rcpp::DataFrame>(sheet_data["cc_out"]);
+  Rcpp::DataFrame cc = Rcpp::as<Rcpp::DataFrame>(sheet_data["cc"]);
 
-
-  xmldoc *doc = new xmldoc;
+  xmldoc* doc = new xmldoc;
   pugi::xml_parse_result result;
 
   pugi::xml_document xml_pr;
@@ -228,10 +221,7 @@ XPtrXML write_worksheet(
   pugi::xml_node sheetData = worksheet.append_child("sheetData");
 
   if (cc.size() > 0) {
-    pugi::xml_document xml_sd;
-    xml_sd = xml_sheet_data(row_attr, cc);
-    for (auto sd : xml_sd.children())
-      sheetData.append_copy(sd);
+    xml_sheet_data(sheetData, row_attr, cc);
   }
 
   if (!post.empty()) {
@@ -254,10 +244,7 @@ XPtrXML write_worksheet(
 }
 
 // [[Rcpp::export]]
-void write_xmlPtr(
-    XPtrXML doc,
-    std::string fl
-) {
+void write_xmlPtr(XPtrXML doc, std::string fl) {
   uint32_t pugi_format_flags = pugi::format_raw | pugi::format_no_escapes;
   const bool success = doc->save_file(fl.c_str(), "", pugi_format_flags, pugi::encoding_utf8);
   if (!success) Rcpp::stop("could not save file");
