@@ -15,19 +15,19 @@ test_that("rbindlist", {
 
 test_that("dims to col & row and back", {
 
-  exp <- list(c("A", "B"), c("1", "2"))
+  exp <- list(col = c("A", "B"), row = c("1", "2"))
   got <- dims_to_rowcol("A1:B2")
   expect_equal(exp, got)
 
-  exp <- list(1:2, 1:2)
+  exp <- list(col = 1:2, row = 1:2)
   got <- dims_to_rowcol("A1:B2", as_integer = TRUE)
   expect_equal(exp, got)
 
-  exp <- list(1:2, c(1L))
+  exp <- list(col = 1:2, row = c(1L))
   got <- dims_to_rowcol("A:B", as_integer = TRUE)
   expect_equal(exp, got)
 
-  exp <- list("A", c("1"))
+  exp <- list(col = "A", row = c("1"))
   got <- dims_to_rowcol("A:A", as_integer = FALSE)
   expect_equal(exp, got)
 
@@ -615,6 +615,11 @@ test_that("fmt_txt works", {
   got <- unclass(txt)
   expect_equal(exp, got)
 
+  expect_equal(
+    fmt_txt("foo", color = wb_color("green")),
+    fmt_txt("foo", colour = wb_colour("green"))
+  )
+
 })
 
 test_that("wb_dims(from_dims) positioning works", {
